<?php

namespace Classifieds\Ad\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Classifieds\Ad\Forms\Ad as AdForm;
use Classifieds\Ad\Http\Requests\AdResourceRequest;
use Classifieds\Ad\Http\Resources\AdResource;
use Classifieds\Ad\Http\Resources\AdsCollection;
use Classifieds\Ad\Models\Ad;
use Classifieds\Ad\Actions\AdAction;
use Classifieds\Ad\Actions\AdActions;


/**
 * Resource controller class for ad.
 */
class AdResourceController extends BaseController
{

    /**
     * Initialize ad resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = AdForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('classifieds.ad.modules'), 'ad', guard_url('ad'));
            return $next($request);
        });
    }

    /**
     * Display a list of ad.
     *
     * @return Response
     */
    public function index(AdResourceRequest $request)
    {
        $request = $request->all();
        $page = AdActions::run('paginate', $request);

        $data = new AdsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('ad::ad.names'))
            ->view('ad::ad.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display ad.
     *
     * @param Request $request
     * @param Model   $ad
     *
     * @return Response
     */
    public function show(AdResourceRequest $request, Ad $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new AdResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('ad::ad.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('ad::ad.show')
            ->output();
    }

    /**
     * Show the form for creating a new ad.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(AdResourceRequest $request, Ad $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new AdResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('ad::ad.name'))
            ->view('ad::ad.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new ad.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(AdResourceRequest $request, Ad $model)
    {
        try {
            $request = $request->all();
            $model = AdAction::run('store', $model, $request);
            $data = new AdResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('ad::ad.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('ad/ad/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/ad/ad'))
                ->redirect();
        }

    }

    /**
     * Show ad for editing.
     *
     * @param Request $request
     * @param Model   $ad
     *
     * @return Response
     */
    public function edit(AdResourceRequest $request, Ad $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new AdResource($model);
        // return view('ad::ad.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('ad::ad.name'))
            ->view('ad::ad.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the ad.
     *
     * @param Request $request
     * @param Model   $ad
     *
     * @return Response
     */
    public function update(AdResourceRequest $request, Ad $model)
    {
        try {
            $request = $request->all();
            $model = AdAction::run('update', $model, $request);
            $data = new AdResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('ad::ad.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('ad/ad/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('ad/ad/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the ad.
     *
     * @param Model   $ad
     *
     * @return Response
     */
    public function destroy(AdResourceRequest $request, Ad $model)
    {
        try {

            $request = $request->all();
            $model = AdAction::run('destroy', $model, $request);
            $data = new AdResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('ad::ad.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('ad/ad/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('ad/ad/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
